Index: res/res_features.c
===================================================================
RCS file: /usr/cvsroot/asterisk/res/res_features.c,v
retrieving revision 1.31
diff -u -r1.31 res_features.c
--- res/res_features.c	6 Jan 2005 17:19:54 -0000	1.31
+++ res/res_features.c	7 Jan 2005 17:38:12 -0000
@@ -365,40 +365,73 @@
 
 static int builtin_automonitor(struct ast_channel *chan, struct ast_channel *peer, struct ast_bridge_config *config, char *code, int sense)
 {
-	char *args;
-	if (option_verbose > 3)
-        ast_verbose(VERBOSE_PREFIX_3 "User hit '%s' to record call.\n", code);
-	if (monitor_ok) {
-		if (!monitor_app) { 
-			if (!(monitor_app = pbx_findapp("Monitor"))) {
-				monitor_ok=0;
-				return -1;
-			}
-		}
-		/* Copy to local variable just in case one of the channels goes away */
-		args = pbx_builtin_getvar_helper(chan, "TOUCH_MONITOR");
-		if (!args)
-			args = pbx_builtin_getvar_helper(peer, "TOUCH_MONITOR");
-		if (!args)
-			args = "WAV||m";
+	char *touch_monitor = NULL, *chan_id = NULL, *peer_id = NULL, *args = NULL;
+	int x = 0;
+	size_t len;
+	
+	if (!monitor_ok) {
+		ast_log(LOG_ERROR,"Cannot record the call. The monitor application is disabled.\n");
+		return -1;
+	}
 
-		pbx_exec(peer, monitor_app, args, 1);
-		if (!ast_strlen_zero(courtesytone)) {
-			if (ast_autoservice_start(peer))
+	if (!monitor_app) { 
+		if (!(monitor_app = pbx_findapp("Monitor"))) {
+			monitor_ok=0;
+			ast_log(LOG_ERROR,"Cannot record the call. The monitor application is disabled.\n");
+			return -1;
+		}
+	}
+	if (!ast_strlen_zero(courtesytone)) {
+		if (ast_autoservice_start(peer))
+			return -1;
+		if (!ast_streamfile(chan, courtesytone, chan->language)) {
+			if (ast_waitstream(chan, "") < 0) {
+				ast_log(LOG_WARNING, "Failed to play courtesy tone!\n");
+				ast_autoservice_stop(peer);
 				return -1;
-			if (!ast_streamfile(chan, courtesytone, chan->language)) {
-				if (ast_waitstream(chan, "") < 0) {
-					ast_log(LOG_WARNING, "Failed to play courtesy tone!\n");
-					ast_autoservice_stop(peer);
-					return -1;
-				}
 			}
-			if (ast_autoservice_stop(peer))
-				return -1;
 		}
+		if (ast_autoservice_stop(peer))
+			return -1;
+	}
+	
+	if (peer->monitor) {
+		if (option_verbose > 3)
+			ast_verbose(VERBOSE_PREFIX_3 "User hit '%s' to stop recording call.\n", code);
+		ast_monitor_stop(peer, 1);
 		return FEATURE_RETURN_SUCCESS;
 	}
 
+	if(chan && peer) {
+		touch_monitor = pbx_builtin_getvar_helper(chan, "TOUCH_MONITOR");
+		if (!touch_monitor)
+			touch_monitor = pbx_builtin_getvar_helper(peer, "TOUCH_MONITOR");
+		
+		if(touch_monitor) {
+			len = strlen(touch_monitor) + 50;
+			args = alloca(len);
+			snprintf(args, len, "WAV|auto-%ld-%s|m", time(NULL), touch_monitor);
+		} else {
+			chan_id = ast_strdupa(chan->cid.cid_num ? chan->cid.cid_num : chan->name);
+			peer_id = ast_strdupa(peer->cid.cid_num ? peer->cid.cid_num : peer->name);
+			len = strlen(chan_id) + strlen(peer_id) + 50;
+			args = alloca(len);
+			snprintf(args, len, "WAV|auto-%ld-%s-%s|m", time(NULL), chan_id, peer_id);
+		}
+
+		for( x = 0; x < strlen(args); x++)
+			if(args[x] == '/')
+				args[x] = '-';
+		
+		if (option_verbose > 3)
+			ast_verbose(VERBOSE_PREFIX_3 "User hit '%s' to record call. filename: %s\n", code, args);
+
+		pbx_exec(peer, monitor_app, args, 1);
+		
+		return FEATURE_RETURN_SUCCESS;
+	}
+	
+	ast_log(LOG_ERROR,"Cannot record the call. One or both channels have gone away.\n");	
 	return -1;
 }
 
